﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Media;
using Kitware.VTK;

namespace HelloVTKWPF
{
  /// <summary>
  /// Logique d'interaction pour MainWindow.xaml
  /// </summary>
  public partial class MainWindow : Window
  {
    public ViewMod mod { get; set; }
  
    public MainWindow()
    {
      InitializeComponent();
      this.Title = "WITHAVI";
			this.LocationChanged += MainWindow_LocationChanged;
		}
    private void MainWindow_LocationChanged(object sender, EventArgs e)
    {
      System.Windows.Forms.Screen screen = System.Windows.Forms.Screen.FromHandle(new System.Windows.Interop.WindowInteropHelper(this).Handle);
      //evaluate the screen variable
    }
    private void Form_Loaded(object sender, EventArgs e)
    {


			AddLine();
			mod = new ViewMod(_renderControl.RenderWindow);
			this.DataContext = mod;

		}

    void AddLine()
		{
      double[] origin = new double[3]{ 0.0, 0.0, 0.0 };
      double[] p0 = new double[3]{ 1.0, 0.0, 0.0 };
      double[] p1 = new double[3]{ 0.0, 1.0, 0.0 };
      double[] p2 = new double[3]{ 0.0, 1.0, 2.0 };
      double[] p3 = new double[3] { 1.0, 2.0, 3.0 };

      // Create a vtkPoints object and store the points in it
      vtkPoints points = new vtkPoints();
      points.InsertNextPoint(0.0, 0.0, 0.0);
      points.InsertNextPoint(1.0, 0.0, 0.0);
      points.InsertNextPoint(0.0, 1.0, 0.0);
      points.InsertNextPoint(0.0, 1.0, 2.0);
      points.InsertNextPoint(1.0, 2.0, 3.0);

      // Create a cell array to store the lines in and add the lines to it
      vtkCellArray lines = new vtkCellArray();

      for (int i = 0; i < 3; i++)
      {
        vtkLine line = new vtkLine();
        line.GetPointIds().SetId(0, i);
        line.GetPointIds().SetId(1, i + 1);
        lines.InsertNextCell(line);
      }

      // Create a polydata to store everything in
      vtkPolyData linesPolyData = new vtkPolyData();

      // Add the points to the dataset
      linesPolyData.SetPoints(points);

      // Add the lines to the dataset
      linesPolyData.SetLines(lines);

      // Setup actor and mapper

      vtkPolyDataMapper mapper = new vtkPolyDataMapper();
      mapper.SetInputData(linesPolyData);

      vtkActor actor = new vtkActor();
      actor.SetMapper(mapper);
      actor.GetProperty().SetLineWidth(4);
      vtkRenderWindow RenWin = _renderControl.RenderWindow;

      vtkRenderer renderer = new vtkRenderer();
      renderer = RenWin.GetRenderers().GetFirstRenderer();
      renderer.SetBackground(.1, .1, .1);

      renderer.AddActor(actor);
      //renderer.SetBackground(colors.GetColor3d("MidnightBlue"));
      renderer.ResetCamera();
    }

    private void Window_Loaded(object sender, RoutedEventArgs e)
    {
			mod.Start();
		}

  }

  public class ViewMod : INotifyPropertyChanged
  {

    public event PropertyChangedEventHandler PropertyChanged;
    double fps;
    double x, y,z;
    string message;
    public int scale = 1;

		public vtkRenderWindowInteractor Interactor;
    public vtkRenderWindow RenWin;
    public vtkPolyDataMapper mapper;
    public vtkRenderer renderer;


    vtkWorldPointPicker worldPointPicker = new vtkWorldPointPicker();
    public vtkActor actor = vtkActor.New();
    vtkOrientationMarkerWidget OAxes;


    public ViewMod(vtkRenderWindow _RenWin)
    {
      RenWin = _RenWin;
      // Create components of the rendering subsystem
      //
      renderer = RenWin.GetRenderers().GetFirstRenderer();
      renderer.SetBackground(.4, .4, .4);
      Interactor = RenWin.GetInteractor();
			Interactor.SetPicker(worldPointPicker);
			Interactor.Initialize();


			//renderer.SetUseDepthPeeling(0);

			//// 4. Set depth peeling parameters
			//// - Set the maximum number of rendering passes (initial value is 4):
			renderer.SetMaximumNumberOfPeels(4);
			//   //// - Set the occlusion ratio (initial value is 0.0, exact image):
			renderer.SetOcclusionRatio(0.1);


			//   ///// Deep Peeling Settings
			RenWin.SetAlphaBitPlanes(1);

			//   //// 2. Force to not pick a framebuffer with a multisample buffer
			//   //// (as initial value is 8):
			//   ///
			RenWin.SetMultiSamples(8);
			renderer.SetUseFXAA(false);
			RenWin.DoubleBufferOn();

			Interactor.SetStillUpdateRate(1000);
			//   renderer.SetUseFXAA(true);

			OAxes = new vtkOrientationMarkerWidget();

			   vtkAxesActor OrientationAxes = new vtkAxesActor();

			   OrientationAxes.SetXAxisLabelText("X");
			   OrientationAxes.SetYAxisLabelText("Y");
			   OrientationAxes.SetZAxisLabelText("Z");

			   OrientationAxes.GetXAxisCaptionActor2D().GetTextActor().SetTextScaleModeToNone();
			   OrientationAxes.GetYAxisCaptionActor2D().GetTextActor().SetTextScaleModeToNone();
			   OrientationAxes.GetZAxisCaptionActor2D().GetTextActor().SetTextScaleModeToNone();

			   //OrientationAxes.GetXAxisCaptionActor2D().SetCaptionTextProperty(TextProp);
			   //OrientationAxes.GetYAxisCaptionActor2D().SetCaptionTextProperty(TextProp);
			   //OrientationAxes.GetZAxisCaptionActor2D().SetCaptionTextProperty(TextProp);

			   OrientationAxes.GetZAxisShaftProperty().SetColor(0.0, 1.0, 0.0);
			   OrientationAxes.GetZAxisTipProperty().SetColor(0.0, 1.0, 0.0);

			   OrientationAxes.GetYAxisShaftProperty().SetColor(1.0, 1.0, 0.0);
			   OrientationAxes.GetYAxisTipProperty().SetColor(1.0, 1.0, 0.0);

			   OAxes.SetOrientationMarker(OrientationAxes);

			   OAxes.SetOutlineColor(0.9300, 0.5700, 0.1300);
			   OAxes.Modified();


			   OAxes.SetInteractor(Interactor);
			OAxes.SetViewport(0.75, 0, 1.0, 0.25);

		}
   
 
    public void Start()
    {
			Interactor.AnyEvt += Interactor_AnyEvt;

			vtkCamera cam = renderer.GetActiveCamera();
      cam.SetParallelProjection(1);
      cam.SetPosition(0.0, 0.0, 1.0);
      cam.SetViewUp(0, 1, 0);

			//hoverWidget.EnabledOn();
			OAxes.SetEnabled(1);
			OAxes.InteractiveOn();
			renderer.ResetCamera();
      RenWin.Render();
      addPlaneWIdget();
    }
    public void Interactor_AnyEvt(vtkObject sender, vtkObjectEventArgs e)
    {
      UpdateCoordinates();
      double timeInSeconds = renderer.GetLastRenderTimeInSeconds();
      FPS = 1.0 / timeInSeconds;
    }
    void UpdateCoordinates()
    {
      //return;
      double[] pt = new double[3];
      IntPtr xyzpnt2 = new IntPtr();
      xyzpnt2 = getPTr(pt);
      int[] pos = Interactor.GetEventPosition();

      vtkWorldPointPicker wpp = new vtkWorldPointPicker();
      wpp.Pick(pos[0], pos[1], 1, renderer);
      double[] wpt = wpp.GetPickPosition();
      if (wpt[0] != 0)
      {
        X = wpt[0];
        Y = wpt[1];
        Z = wpt[2] / scale;
      }
      else
      {
        double z = renderer.GetZ((int)x, (int)y);
        vtkCoordinate coordinate = new vtkCoordinate();
        coordinate.SetCoordinateSystemToDisplay();
        coordinate.SetValue(x, y, z);
        pt = coordinate.GetComputedWorldValue(renderer);
        X = pt[0];
        Y = pt[1];
        Z = pt[2] / scale;
      }
    }
    void addPlaneWIdget()
    {

      renderer.ResetCamera();
      RenWin.Render();


      vtkFinitePlaneRepresentation fpr = new vtkFinitePlaneRepresentation();
      fpr.SetTubing(true);
      fpr.SetDrawPlane(true);
      fpr.SetHandles(true);

      double[] bounds = new double[6] { 0, 1, 0, 1, 0, 1 };
      IntPtr ptrp1 = getPTr(bounds);
      fpr.PlaceWidget(ptrp1);

      fpr.SetPlaceFactor(1); // This must be set prior to placing the widget
      fpr.GetPlaneProperty().SetOpacity(0.5);
      //fpr.SetOutlineTranslation(0);

      vtkFinitePlaneWidget finitePlaneWidget = new vtkFinitePlaneWidget();
      finitePlaneWidget.SetInteractor(RenWin.GetInteractor());
      //finitePlaneWidget.SetCurrentRenderer(CurrentRenderer);
      finitePlaneWidget.SetRepresentation(fpr);
      finitePlaneWidget.On();

      finitePlaneWidget.SetEnabled(1);

      renderer.ResetCamera();
      RenWin.Render();
    }

    public double X
    {
      get
      {
        return x;
      }
      set
      {
        x = value;
        OnPropertyChanged("X");

      }
    }

    public double FPS
    {
      get
      {
        return fps;
      }
      set
      {
        fps = value;


        OnPropertyChanged("FPS");
        //actor.GetProperty().SetOpacity(0.8);
      }
    }
    public double Y
    {
      get
      {
        return y;
      }
      set
      {
        y = value;
        OnPropertyChanged("Y");

      }
    }
    public double Z
    {
      get
      {
        return z;
      }
      set
      {
        z = value;
        OnPropertyChanged("Z");

      }
    }
  
   

    public void OnPropertyChanged(string propertyName)
    {
      PropertyChangedEventHandler handler = PropertyChanged;
      if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
    }

  
    public static IntPtr getPTr(double[] point)
    {
      int size = Marshal.SizeOf(point[0]) * point.Length;
      IntPtr Pointpnt = Marshal.AllocHGlobal(size);
      Marshal.Copy(point, 0, Pointpnt, point.Length);
      return Pointpnt;
    }

 

  }
}
